/* Programul calculeaza integrala (in sens de valoare principala Cauchy) functiei f(x)
 * pe (a, b), cu o singularitate in c:
 * \int_a^b {f(x)/(x-c) dx}=\lim_{\epsilon \to 0}{\int_a^{c-\epsilon}{f(x)/(x-c) dx}+\int_{c+\epsilon}^b{f(x)/(x-c) dx}}
 * Este utilizat algoritmul QAWC, implementat in biblioteca numerică GSL. Algoritmul adaptativ asigura faptul ca 
 * marginile subdiviziunilor intervalului (a,b) nu coincid cu punctul x = c.
 */

#include <stdio.h>
#include <math.h>
#include <gsl/gsl_integration.h>

// functia f(x)
double f (double x, void * params) 
{
    double alpha = *(double *) params;
    double f = exp(-alpha*(x-5.)*(x-5.)) * sqrt(x);
    return f;
}
        
int main (void)
{
    // spatiul alocat dinamic pentru dezvoltarea algoritmului
    gsl_integration_workspace * w = gsl_integration_workspace_alloc (1000); 
    // result contine rezultatul, error contine eroarea estimata
    double result, error;
    
    // parametrul de transmis functiei f(x)
    double alpha = 0.1;
    
    gsl_function F;
    F.function = &f;
    F.params = &alpha;
    
    // integrarea se face pe intervalul (1,10), punctul singular fiind c=4.35    
    gsl_integration_qawc (&F , 1. , 10. , 4.35 , 0. , 1e-7 , 1000 , w, &result, &error);
    
    printf("rezultatul = %.18f\n", result);
    printf("eroarea estimata = %.18f\n", error);
    // numarul de subintervale utilizat de algoritm
    printf("intervale = %d\n",w->size);
    
    return 0;
}

